var StringIterator = (function () {
    'use strict';
    /**
     * @description Create a StringIterator.
     * @typedef StringIterator This gives you the possibility to navigate into strings.
     * @param {String} string
     * @param {String} [stringSeparator] Specify a character used to split the string. Default = " ".
     * @constructor
     */
    function StringIterator (string, stringSeparator) {
        stringSeparator = stringSeparator || " ";
        /**
         * @description Contains all separated string.
         * @type {Array<String>}
         * @public
         */
        this.string_arr = string.split(stringSeparator);
        /**
         * @description The index of the string.
         * @type {number}
         * @public
         */
        this.string_index = 0;
    }

    StringIterator.prototype = {
        /**
         * @description Place the index before the first string.
         */
        beforeFirst :  function () {
            this.string_index = -1;
        },
        /**
         * @description Give the first string.
         * @returns {String}
         */
        first: function () {
            return this.string_arr[(this.string_index = 0)];
        },
        /**
         * @description Give the previous string.
         * @returns {String}
         */
        previous : function () {
            return this.string_arr[--this.string_index];
        },
        /**
         * @description Give the current string.
         * @returns {String}
         */
        current : function () {
            return this.string_arr[this.string_index];
        },
        /**
         * @description Give the next string.
         * @returns {String}
         */
        next : function () {
            return this.string_arr[++this.string_index];
        },
        /**
         * @description Give the last string.
         * @returns {String}
         */
        last : function () {
            return this.string_arr[(this.string_index = this.string_arr.length - 1)];
        },
        /**
         * @description Place the index after the last string.
         */
        afterLast : function () {
            this.string_index = this.string_arr.length;
        },
        /**
         * @description Tell if it has previous strings.
         * @returns {boolean}
         */
        hasPrevious : function () {
            return this.string_index > 0;
        },
        /**
         * @description Tell if it has next strings.
         * @returns {boolean}
         */
        hasNext : function () {
            return this.string_index < this.string_arr.length - 1;
        },
        /**
         * @description Do something for each string it has.
         * @param {Function} callback Function you want to call for each strings
         * @param {Function} [progressionCallback] Function is called each 100 iterations.
         */
        forEach : function (callback, progressionCallback) {
            if (this.string_arr.length > 0) {
                var hasToCallProgressionCallback = (progressionCallback && this.string_arr.length > 100);
                this.beforeFirst();
                do {
                    callback(this.next());
                    if (hasToCallProgressionCallback && (this.string_index % 100 === 0))
                        progressionCallback(this.string_index * 100 / this.string_arr.length);
                } while (this.hasNext());
            }
        }
    };

    return StringIterator;
}());